@@ -23,14 +23,15 @@
 
 #include "fs/filesystem.h"
 #include "fs/filesystemfactory.h"
+#include "fs/luks.h"
 
 #include "util/externalcommand.h"
 #include "util/report.h"
 
-#include <QDebug>
 #include <QRegularExpression>
 #include <QString>
 #include <QStringList>
+#include <QTextStream>
 
 #include <KLocalizedString>
 #include <KIOCore/KMountPoint>
@@ -62,11 +63,11 @@ Partition::Partition(PartitionNode* parent, const Device& device, const Partitio
     m_AvailableFlags(availableFlags),
     m_ActiveFlags(activeFlags),
     m_IsMounted(mounted),
-    m_SectorSize(device.logicalSectorSize()),
     m_State(state)
 {
     setPartitionPath(partitionPath);
     Q_ASSERT(m_Parent);
+    m_SectorSize = device.logicalSize();
 }
 
 /** Destroys a Partition, destroying its children and its FileSystem */
@@ -106,7 +107,7 @@ Partition::Partition(const Partition& other) :
     m_State(other.m_State)
 {
     setPartitionPath(other.m_PartitionPath);
-    foreach(const Partition * child, other.children()) {
+    for (const auto &child : other.children()) {
         Partition* p = new Partition(*child);
         p->setParent(this);
         m_Children.append(p);
@@ -121,7 +122,7 @@ Partition& Partition::operator=(const Partition& other)
 
     clearChildren();
 
-    foreach(const Partition * child, other.children()) {
+    for (const auto &child : other.children()) {
         Partition* p = new Partition(*child);
         p->setParent(this);
         m_Children.append(p);
@@ -158,16 +159,16 @@ bool Partition::operator!=(const Partition& other) const
 QString Partition::deviceNode() const
 {
     if (roles().has(PartitionRole::None) || roles().has(PartitionRole::Unallocated))
-        return i18nc("@item partition name", "unallocated");
+        return xi18nc("@item partition name", "unallocated");
 
     if (state() == StateNew)
-        return i18nc("@item partition name", "New Partition");
+        return xi18nc("@item partition name", "New Partition");
 
     if (state() == StateRestore)
-        return i18nc("@item partition name", "Restored Partition");
+        return xi18nc("@item partition name", "Restored Partition");
 
     if (state() == StateCopy)
-        return i18nc("@item partition name", "Copy of %1", partitionPath());
+        return xi18nc("@item partition name", "Copy of %1", partitionPath());
 
     return partitionPath();
 }
@@ -175,11 +176,15 @@ QString Partition::deviceNode() const
 /** @return the sectors used in the Partition's FileSystem or, in case of an extended partition, the sum of used sectors of the Partition's children */
 qint64 Partition::sectorsUsed() const
 {
+    // Make sure file system exists. In some cases (due to bugs elsewhere?) file system pointer did not exist, especially for unallocated space.
+    if (m_FileSystem == nullptr)
+        return -1;
+
     if (!roles().has(PartitionRole::Extended))
         return fileSystem().sectorsUsed();
 
     qint64 result = 0;
-    foreach(const Partition * p, children())
+    for (const auto &p : children())
         if (!p->roles().has(PartitionRole::Unallocated))
             result += p->length();
 
@@ -190,7 +195,7 @@ qint64 Partition::sectorsUsed() const
 qint64 Partition::minimumSectors() const
 {
     if (roles().has(PartitionRole::Luks))
-        return fileSystem().minCapacity() / sectorSize() + 4096; // 4096 is the default cryptsetup payload offset
+        return ( fileSystem().minCapacity() + (4096 * 512) ) / sectorSize(); // 4096 is the default cryptsetup payload offset
     return fileSystem().minCapacity() / sectorSize();
 }
 
@@ -210,12 +215,12 @@ qint64 Partition::maximumSectors() const
     @param deletedNumber the number of a deleted logical or -1 if none has been deleted
     @param insertedNumber the number of an inserted logical or -1 if none has been inserted
 */
-void Partition::adjustLogicalNumbers(qint32 deletedNumber, qint32 insertedNumber)
+void Partition::adjustLogicalNumbers(qint32 deletedNumber, qint32 insertedNumber) const
 {
     if (!roles().has(PartitionRole::Extended))
         return;
 
-    foreach(Partition * p, children()) {
+    for (const auto &p : children()) {
         QString path = p->partitionPath();
         path.remove(QRegularExpression(QStringLiteral("(\\d+$)")));
         if (deletedNumber > 4 && p->number() > deletedNumber)
@@ -230,7 +235,7 @@ qint64 Partition::maxFirstSector() const
 {
     qint64 rval = -1;
 
-    foreach(const Partition * child, children())
+    for (const auto &child : children())
         if (!child->roles().has(PartitionRole::Unallocated) && (child->firstSector() < rval || rval == -1))
             rval = child->firstSector();
 
@@ -242,7 +247,7 @@ qint64 Partition::minLastSector() const
 {
     qint64 rval = -1;
 
-    foreach(const Partition * child, children())
+    for (const auto &child : children())
         if (!child->roles().has(PartitionRole::Unallocated) && child->lastSector() > rval)
             rval = child->lastSector();
 
@@ -252,7 +257,7 @@ qint64 Partition::minLastSector() const
 /** @return true if the Partition has children */
 bool Partition::hasChildren() const
 {
-    foreach(const Partition * child, children())
+    for (const auto &child : children())
         if (!child->roles().has(PartitionRole::Unallocated))
             return true;
 
@@ -283,7 +288,13 @@ bool Partition::canMount() const
 /** @return true if this Partition can be unmounted */
 bool Partition::canUnmount() const
 {
-    return !roles().has(PartitionRole::Extended) && isMounted();
+    return !roles().has(PartitionRole::Extended) && isMounted() && fileSystem().canUnmount(deviceNode());
+}
+
+void Partition::setMounted(bool b) {
+    m_IsMounted = b;
+    if (roles().has(PartitionRole::Luks))
+        static_cast<FS::luks*>(m_FileSystem)->setMounted(b);
 }
 
 /** Tries to mount a Partition.
@@ -365,7 +376,7 @@ QTextStream& operator<<(QTextStream& stream, const Partition& p)
 {
     QStringList flagList;
 
-    foreach(const PartitionTable::Flag & f, PartitionTable::flagList()) {
+    for (const auto &f : PartitionTable::flagList()) {
         if (p.activeFlags() & f)
             flagList.append(PartitionTable::flagName(f));
     }
