@@ -21,6 +21,7 @@
 #include "core/partition.h"
 #include "core/partitiontable.h"
 #include "core/device.h"
+#include "core/diskdevice.h"
 
 #include "fs/filesystem.h"
 
@@ -33,11 +34,12 @@ int PartitionAlignment::s_sectorAlignment = 2048;
 qint64 PartitionAlignment::firstDelta(const Device& d, const Partition& p, qint64 s)
 {
     if (d.partitionTable()->type() == PartitionTable::msdos) {
-        if (p.roles().has(PartitionRole::Logical) && s == 2 * d.sectorsPerTrack())
-            return (s - (2 * d.sectorsPerTrack())) % sectorAlignment(d);
+        const DiskDevice& diskDevice = dynamic_cast<const DiskDevice&>(d);
+        if (p.roles().has(PartitionRole::Logical) && s == 2 * diskDevice.sectorsPerTrack())
+            return (s - (2 * diskDevice.sectorsPerTrack())) % sectorAlignment(d);
 
-        if (p.roles().has(PartitionRole::Logical) || s == d.sectorsPerTrack())
-            return (s - d.sectorsPerTrack()) % sectorAlignment(d);
+        if (p.roles().has(PartitionRole::Logical) || s == diskDevice.sectorsPerTrack())
+            return (s - diskDevice.sectorsPerTrack()) % sectorAlignment(d);
     }
 
     return s % sectorAlignment(d);
@@ -51,11 +53,12 @@ qint64 PartitionAlignment::lastDelta(const Device& d, const Partition&, qint64 s
 bool PartitionAlignment::isLengthAligned(const Device& d, const Partition& p)
 {
     if (d.partitionTable()->type() == PartitionTable::msdos) {
-        if (p.roles().has(PartitionRole::Logical) && p.firstSector() == 2 * d.sectorsPerTrack())
-            return (p.length() + (2 * d.sectorsPerTrack())) % sectorAlignment(d) == 0;
+        const DiskDevice& diskDevice = dynamic_cast<const DiskDevice&>(d);
+        if (p.roles().has(PartitionRole::Logical) && p.firstSector() == 2 * diskDevice.sectorsPerTrack())
+            return (p.length() + (2 * diskDevice.sectorsPerTrack())) % sectorAlignment(d) == 0;
 
-        if (p.roles().has(PartitionRole::Logical) || p.firstSector() == d.sectorsPerTrack())
-            return (p.length() + d.sectorsPerTrack()) % sectorAlignment(d) == 0;
+        if (p.roles().has(PartitionRole::Logical) || p.firstSector() == diskDevice.sectorsPerTrack())
+            return (p.length() + diskDevice.sectorsPerTrack()) % sectorAlignment(d) == 0;
     }
 
     return p.length() % sectorAlignment(d) == 0;
@@ -82,10 +85,10 @@ bool PartitionAlignment::isAligned(const Device& d, const Partition& p, bool qui
 bool PartitionAlignment::isAligned(const Device& d, const Partition& p, qint64 newFirst, qint64 newLast, bool quiet)
 {
     if (firstDelta(d, p, newFirst) && !quiet)
-        Log(Log::warning) << xi18nc("@info/plain", "Partition <filename>%1</filename> is not properly aligned (first sector: %2, modulo: %3).", p.deviceNode(), newFirst, firstDelta(d, p, newFirst));
+        Log(Log::warning) << xi18nc("@info:status", "Partition <filename>%1</filename> is not properly aligned (first sector: %2, modulo: %3).", p.deviceNode(), newFirst, firstDelta(d, p, newFirst));
 
     if (lastDelta(d, p, newLast) && !quiet)
-        Log(Log::warning) << xi18nc("@info/plain", "Partition <filename>%1</filename> is not properly aligned (last sector: %2, modulo: %3).", p.deviceNode(), newLast, lastDelta(d, p, newLast));
+        Log(Log::warning) << xi18nc("@info:status", "Partition <filename>%1</filename> is not properly aligned (last sector: %2, modulo: %3).", p.deviceNode(), newLast, lastDelta(d, p, newLast));
 
     return firstDelta(d, p, newFirst) == 0 && lastDelta(d, p, newLast) == 0;
 }
