@@ -23,8 +23,9 @@
 #include "core/partitiontable.h"
 #include "core/partition.h"
 #include "core/device.h"
+#include "core/diskdevice.h"
+#include "core/lvmdevice.h"
 #include "core/partitionalignment.h"
-
 #include "fs/filesystem.h"
 #include "fs/filesystemfactory.h"
 
@@ -38,16 +39,34 @@
 /** Creates a new PartitionTable object with type MSDOS
     @param type name of the PartitionTable type (e.g. "msdos" or "gpt")
 */
-PartitionTable::PartitionTable(TableType type, qint64 first_usable, qint64 last_usable) :
-    PartitionNode(),
-    m_Children(),
-    m_MaxPrimaries(maxPrimariesForTableType(type)),
-    m_Type(type),
-    m_FirstUsable(first_usable),
-    m_LastUsable(last_usable)
+PartitionTable::PartitionTable(TableType type, qint64 firstUsable, qint64 lastUsable)
+    : PartitionNode()
+    , m_Children()
+    , m_MaxPrimaries(maxPrimariesForTableType(type))
+    , m_Type(type)
+    , m_FirstUsable(firstUsable)
+    , m_LastUsable(lastUsable)
 {
 }
 
+/** Copy constructor for PartitionTable.
+ * @param other the other PartitionTable.
+ */
+PartitionTable::PartitionTable(const PartitionTable& other)
+    : PartitionNode()
+    , m_Children()
+    , m_MaxPrimaries(other.m_MaxPrimaries)
+    , m_Type(other.m_Type)
+    , m_FirstUsable(other.m_FirstUsable)
+    , m_LastUsable(other.m_LastUsable)
+{
+    for (Partitions::const_iterator it = other.m_Children.constBegin();
+         it != other.m_Children.constEnd(); ++it)
+    {
+        m_Children.append(new Partition(**it));
+    }
+}
+
 /** Destroys a PartitionTable object, destroying all children */
 PartitionTable::~PartitionTable()
 {
@@ -88,11 +107,23 @@ qint64 PartitionTable::freeSectorsAfter(const Partition& p) const
     return 0;
 }
 
+qint64 PartitionTable::freeSectors() const
+{
+    qint64 sectors = 0;
+    for (const auto &p : children()) {
+        if (p->roles().has(PartitionRole::Unallocated)) {
+            sectors += p->length();
+        }
+    }
+
+    return sectors;
+}
+
 /** @return true if the PartitionTable has an extended Partition */
 bool PartitionTable::hasExtended() const
 {
-    for (int i = 0; i < children().size(); i++)
-        if (children()[i]->roles().has(PartitionRole::Extended))
+    for (const auto &p : children())
+        if (p->roles().has(PartitionRole::Extended))
             return true;
 
     return false;
@@ -101,9 +132,9 @@ bool PartitionTable::hasExtended() const
 /** @return pointer to the PartitionTable's extended Partition or nullptr if none exists */
 Partition* PartitionTable::extended() const
 {
-    for (int i = 0; i < children().size(); i++)
-        if (children()[i]->roles().has(PartitionRole::Extended))
-            return children()[i];
+    for (const auto &p : children())
+        if (p->roles().has(PartitionRole::Extended))
+            return p;
 
     return nullptr;
 }
@@ -129,7 +160,7 @@ int PartitionTable::numPrimaries() const
 {
     int result = 0;
 
-    foreach(const Partition * p, children())
+    for (const auto &p : children())
         if (p->roles().has(PartitionRole::Primary) || p->roles().has(PartitionRole::Extended))
             result++;
 
@@ -151,41 +182,41 @@ QString PartitionTable::flagName(Flag f)
 {
     switch (f) {
     case PartitionTable::FlagBoot:
-        return i18nc("@item partition flag", "boot");
+        return xi18nc("@item partition flag", "boot");
     case PartitionTable::FlagRoot:
-        return i18nc("@item partition flag", "root");
+        return xi18nc("@item partition flag", "root");
     case PartitionTable::FlagSwap:
-        return i18nc("@item partition flag", "swap");
+        return xi18nc("@item partition flag", "swap");
     case PartitionTable::FlagHidden:
-        return i18nc("@item partition flag", "hidden");
+        return xi18nc("@item partition flag", "hidden");
     case PartitionTable::FlagRaid:
-        return i18nc("@item partition flag", "raid");
+        return xi18nc("@item partition flag", "raid");
     case PartitionTable::FlagLvm:
-        return i18nc("@item partition flag", "lvm");
+        return xi18nc("@item partition flag", "lvm");
     case PartitionTable::FlagLba:
-        return i18nc("@item partition flag", "lba");
+        return xi18nc("@item partition flag", "lba");
     case PartitionTable::FlagHpService:
-        return i18nc("@item partition flag", "hpservice");
+        return xi18nc("@item partition flag", "hpservice");
     case PartitionTable::FlagPalo:
-        return i18nc("@item partition flag", "palo");
+        return xi18nc("@item partition flag", "palo");
     case PartitionTable::FlagPrep:
-        return i18nc("@item partition flag", "prep");
+        return xi18nc("@item partition flag", "prep");
     case PartitionTable::FlagMsftReserved:
-        return i18nc("@item partition flag", "msft-reserved");
+        return xi18nc("@item partition flag", "msft-reserved");
     case PartitionTable::FlagBiosGrub:
-        return i18nc("@item partition flag", "bios-grub");
+        return xi18nc("@item partition flag", "bios-grub");
     case PartitionTable::FlagAppleTvRecovery:
-        return i18nc("@item partition flag", "apple-tv-recovery");
+        return xi18nc("@item partition flag", "apple-tv-recovery");
     case PartitionTable::FlagDiag:
-        return i18nc("@item partition flag", "diag");
+        return xi18nc("@item partition flag", "diag");
     case PartitionTable::FlagLegacyBoot:
-        return i18nc("@item partition flag", "legacy-boot");
+        return xi18nc("@item partition flag", "legacy-boot");
     case PartitionTable::FlagMsftData:
-        return i18nc("@item partition flag", "msft-data");
+        return xi18nc("@item partition flag", "msft-data");
     case PartitionTable::FlagIrst:
-        return i18nc("@item partition flag", "irst");
+        return xi18nc("@item partition flag", "irst");
     case PartitionTable::FlagEsp:
-        return i18nc("@item partition flag", "esp");
+        return xi18nc("@item partition flag", "esp");
     default:
         break;
     }
@@ -194,7 +225,7 @@ QString PartitionTable::flagName(Flag f)
 }
 
 /** @return list of all flags */
-QList<PartitionTable::Flag> PartitionTable::flagList()
+const QList<PartitionTable::Flag> PartitionTable::flagList()
 {
     QList<PartitionTable::Flag> rval;
 
@@ -240,29 +271,38 @@ QStringList PartitionTable::flagNames(Flags flags)
     return rval;
 }
 
-bool PartitionTable::getUnallocatedRange(const Device& device, PartitionNode& parent, qint64& start, qint64& end)
+bool PartitionTable::getUnallocatedRange(const Device& d, PartitionNode& parent, qint64& start, qint64& end)
 {
-    if (!parent.isRoot()) {
-        Partition* extended = dynamic_cast<Partition*>(&parent);
+    if (d.type() == Device::Disk_Device) {
+        const DiskDevice& device = dynamic_cast<const DiskDevice&>(d);
+        if (!parent.isRoot()) {
+            Partition* extended = dynamic_cast<Partition*>(&parent);
 
-        if (extended == nullptr) {
-            qWarning() << "extended is null. start: " << start << ", end: " << end << ", device: " << device.deviceNode();
-            return false;
+            if (extended == nullptr) {
+                qWarning() << "extended is null. start: " << start << ", end: " << end << ", device: " << device.deviceNode();
+                return false;
+            }
+
+            // Leave a track (cylinder aligned) or sector alignment sectors (sector based) free at the
+            // start for a new partition's metadata
+            start += device.partitionTable()->type() == PartitionTable::msdos ? device.sectorsPerTrack() : PartitionAlignment::sectorAlignment(device);
+
+            // .. and also at the end for the metadata for a partition to follow us, if we're not
+            // at the end of the extended partition
+            if (end < extended->lastSector())
+                end -= device.partitionTable()->type() == PartitionTable::msdos ? device.sectorsPerTrack() : PartitionAlignment::sectorAlignment(device);
         }
 
-        // Leave a track (cylinder aligned) or sector alignment sectors (sector based) free at the
-        // start for a new partition's metadata
-        start += device.partitionTable()->type() == PartitionTable::msdos ? device.sectorsPerTrack() : PartitionAlignment::sectorAlignment(device);
-
-        // .. and also at the end for the metadata for a partition to follow us, if we're not
-        // at the end of the extended partition
-        if (end < extended->lastSector())
-            end -= device.partitionTable()->type() == PartitionTable::msdos ? device.sectorsPerTrack() : PartitionAlignment::sectorAlignment(device);
+        return end - start + 1 >= PartitionAlignment::sectorAlignment(device);
+    } else if (d.type() == Device::LVM_Device) {
+        if (end - start + 1 > 0) {
+            return true;
+        }
     }
-
-    return end - start + 1 >= PartitionAlignment::sectorAlignment(device);
+    return false;
 }
 
+
 /** Creates a new unallocated Partition on the given Device.
     @param device the Device to create the new Partition on
     @param parent the parent PartitionNode for the new Partition
@@ -334,13 +374,26 @@ void PartitionTable::insertUnallocated(const Device& d, PartitionNode* p, qint64
 
     qint64 lastEnd = start;
 
-    foreach(Partition * child, p->children()) {
-        p->insert(createUnallocated(d, *p, lastEnd, child->firstSector() - 1));
+    if (d.type() == Device::LVM_Device && !p->children().isEmpty()) {
+        // rearranging all the partitions sector to keep all the unallocated space at the end
+        lastEnd = 0;
+        for (const auto &child : children()) {
+            qint64 totalSector = child->length();
+            child->setFirstSector(lastEnd);
+            child->setLastSector(lastEnd + totalSector - 1);
 
-        if (child->roles().has(PartitionRole::Extended))
-            insertUnallocated(d, child, child->firstSector());
+            lastEnd += totalSector;
+        }
+    } else {
+        const auto pChildren = p->children();
+        for (const auto &child : pChildren) {
+            p->insert(createUnallocated(d, *p, lastEnd, child->firstSector() - 1));
 
-        lastEnd = child->lastSector() + 1;
+            if (child->roles().has(PartitionRole::Extended))
+                insertUnallocated(d, child, child->firstSector());
+
+            lastEnd = child->lastSector() + 1;
+        }
     }
 
     // Take care of the free space between the end of the last child and the end
@@ -353,7 +406,7 @@ void PartitionTable::insertUnallocated(const Device& d, PartitionNode* p, qint64
         Q_ASSERT(extended);
     }
 
-    if (parentEnd >= firstUsable())
+    if (parentEnd >= firstUsable() && parentEnd >= lastEnd)
         p->insert(createUnallocated(d, *p, lastEnd, parentEnd));
 }
 
@@ -369,15 +422,20 @@ void PartitionTable::updateUnallocated(const Device& d)
 qint64 PartitionTable::defaultFirstUsable(const Device& d, TableType t)
 {
     Q_UNUSED(t)
-    return PartitionAlignment::sectorAlignment(d);
+    if (d.type() == Device::LVM_Device) {
+        return 0;
+    }
+
+    const DiskDevice& diskDevice = dynamic_cast<const DiskDevice&>(d);
+    return PartitionAlignment::sectorAlignment(diskDevice);
 }
 
 qint64 PartitionTable::defaultLastUsable(const Device& d, TableType t)
 {
     if (t == gpt)
-        return d.totalSectors() - 1 - 32 - 1;
+        return d.totalLogical() - 1 - 32 - 1;
 
-    return d.totalSectors() - 1;
+    return d.totalLogical() - 1;
 }
 
 static struct {
@@ -398,50 +456,51 @@ static struct {
     { QStringLiteral("mac"), 0xffff, false, true, PartitionTable::mac },
     { QStringLiteral("pc98"), 16, false, true, PartitionTable::pc98 },
     { QStringLiteral("amiga"), 128, false, true, PartitionTable::amiga },
-    { QStringLiteral("sun"), 8, false, true, PartitionTable::sun }
+    { QStringLiteral("sun"), 8, false, true, PartitionTable::sun },
+    { QStringLiteral("vmd"), 0xffff, false, false, PartitionTable::vmd }
 };
 
 PartitionTable::TableType PartitionTable::nameToTableType(const QString& n)
 {
-    for (size_t i = 0; i < sizeof(tableTypes) / sizeof(tableTypes[0]); i++)
-        if (n == tableTypes[i].name)
-            return tableTypes[i].type;
+    for (const auto &type : tableTypes)
+        if (n == type.name)
+            return type.type;
 
     return PartitionTable::unknownTableType;
 }
 
 QString PartitionTable::tableTypeToName(TableType l)
 {
-    for (size_t i = 0; i < sizeof(tableTypes) / sizeof(tableTypes[0]); i++)
-        if (l == tableTypes[i].type)
-            return tableTypes[i].name;
+    for (const auto &type : tableTypes)
+        if (l == type.type)
+            return type.name;
 
-    return i18nc("@item partition table name", "unknown");
+    return xi18nc("@item partition table name", "unknown");
 }
 
 qint64 PartitionTable::maxPrimariesForTableType(TableType l)
 {
-    for (size_t i = 0; i < sizeof(tableTypes) / sizeof(tableTypes[0]); i++)
-        if (l == tableTypes[i].type)
-            return tableTypes[i].maxPrimaries;
+    for (const auto &type : tableTypes)
+        if (l == type.type)
+            return type.maxPrimaries;
 
     return 1;
 }
 
 bool PartitionTable::tableTypeSupportsExtended(TableType l)
 {
-    for (size_t i = 0; i < sizeof(tableTypes) / sizeof(tableTypes[0]); i++)
-        if (l == tableTypes[i].type)
-            return tableTypes[i].canHaveExtended;
+    for (const auto &type : tableTypes)
+        if (l == type.type)
+            return type.canHaveExtended;
 
     return false;
 }
 
 bool PartitionTable::tableTypeIsReadOnly(TableType l)
 {
-    for (size_t i = 0; i < sizeof(tableTypes) / sizeof(tableTypes[0]); i++)
-        if (l == tableTypes[i].type)
-            return tableTypes[i].isReadOnly;
+    for (const auto &type : tableTypes)
+        if (l == type.type)
+            return type.isReadOnly;
 
     return false;
 }
@@ -452,26 +511,32 @@ bool PartitionTable::tableTypeIsReadOnly(TableType l)
 */
 bool PartitionTable::isSectorBased(const Device& d) const
 {
-    if (type() == PartitionTable::msdos) {
-        // the default for empty partition tables is sector based
-        if (numPrimaries() == 0)
-            return true;
+    if (d.type() == Device::Disk_Device) {
+        const DiskDevice& diskDevice = dynamic_cast<const DiskDevice&>(d);
 
-        quint32 numCylinderAligned = 0;
-        quint32 numSectorAligned = 0;
+        if (type() == PartitionTable::msdos) {
+            // the default for empty partition tables is sector based
+            if (numPrimaries() == 0)
+                return true;
 
-        // see if we have more cylinder aligned partitions than sector
-        // aligned ones.
-        foreach(const Partition * p, children())
-            if (p->firstSector() % PartitionAlignment::sectorAlignment(d) == 0)
-                numSectorAligned++;
-            else if (p->firstSector() % d.cylinderSize() == 0)
-                numCylinderAligned++;
+            quint32 numCylinderAligned = 0;
+            quint32 numSectorAligned = 0;
 
-        return numSectorAligned >= numCylinderAligned;
+            // see if we have more cylinder aligned partitions than sector
+            // aligned ones.
+            for (const auto &p : children()) {
+                if (p->firstSector() % PartitionAlignment::sectorAlignment(diskDevice) == 0)
+                    numSectorAligned++;
+                else if (p->firstSector() % diskDevice.cylinderSize() == 0)
+                    numCylinderAligned++;
+            }
+
+            return numSectorAligned >= numCylinderAligned;
+        }
+        return type() == PartitionTable::msdos_sectorbased;
     }
 
-    return type() == PartitionTable::msdos_sectorbased;
+    return false;
 }
 
 void PartitionTable::setType(const Device& d, TableType t)
@@ -497,19 +562,21 @@ QTextStream& operator<<(QTextStream& stream, const PartitionTable& ptable)
 
     QList<const Partition*> partitions;
 
-    foreach(const Partition * p, ptable.children())
+    for (const auto &p : ptable.children()) {
         if (!p->roles().has(PartitionRole::Unallocated)) {
             partitions.append(p);
 
             if (p->roles().has(PartitionRole::Extended))
-                foreach(const Partition * child, p->children())
-                if (!child->roles().has(PartitionRole::Unallocated))
-                    partitions.append(child);
+                for (const auto &child : p->children()) {
+                    if (!child->roles().has(PartitionRole::Unallocated))
+                        partitions.append(child);
+                }
         }
+    }
 
     qSort(partitions.begin(), partitions.end(), isPartitionLessThan);
 
-    foreach(const Partition * p, partitions)
+    foreach(const auto &p, partitions)
         stream << *p;
 
     return stream;
