@@ -20,13 +20,13 @@
 
 #define PARTITIONTABLE__H
 
-#include "../util/libpartitionmanagerexport.h"
+#include "util/libpartitionmanagerexport.h"
 
-#include "../core/partitionnode.h"
-#include "../core/partitionrole.h"
+#include "core/partitionnode.h"
+#include "core/partitionrole.h"
 
 #include <QList>
-#include <qglobal.h>
+#include <QtGlobal>
 
 class Device;
 class Partition;
@@ -44,7 +44,7 @@ class QTextStream;
 */
 class LIBKPMCORE_EXPORT PartitionTable : public PartitionNode
 {
-    Q_DISABLE_COPY(PartitionTable)
+    PartitionTable &operator=(const PartitionTable &) = delete;
 
     friend class CoreBackend;
     friend LIBKPMCORE_EXPORT QTextStream& operator<<(QTextStream& stream, const PartitionTable& ptable);
@@ -64,7 +64,8 @@ public:
         mac,
         pc98,
         amiga,
-        sun
+        sun,
+        vmd  /* Volume Manager Device */
     };
 
     /** Partition flags */
@@ -91,39 +92,42 @@ public:
     };
 
     Q_DECLARE_FLAGS(Flags, Flag)
+    Q_FLAG(Flag)
 
 public:
-    PartitionTable(TableType type, qint64 first_usable, qint64 last_usable);
+    PartitionTable(TableType type, qint64 firstUsable, qint64 lastUsable);
+    PartitionTable(const PartitionTable& other);
     ~PartitionTable();
 
 public:
-    PartitionNode* parent() {
+    PartitionNode* parent() override {
         return nullptr;    /**< @return always nullptr for PartitionTable */
     }
-    const PartitionNode* parent() const {
+    const PartitionNode* parent() const override {
         return nullptr;    /**< @return always nullptr for PartitionTable */
     }
 
-    bool isRoot() const {
+    bool isRoot() const override {
         return true;    /**< @return always true for PartitionTable */
     }
     bool isReadOnly() const {
         return tableTypeIsReadOnly(type());    /**< @return true if the PartitionTable is read only */
     }
 
-    Partitions& children() {
+    Partitions& children() override {
         return m_Children;    /**< @return the children in this PartitionTable */
     }
-    const Partitions& children() const {
+    const Partitions& children() const override {
         return m_Children;    /**< @return the children in this PartitionTable */
     }
 
     void setType(const Device& d, TableType t);
 
-    void append(Partition* partition);
+    void append(Partition* partition) override;
 
     qint64 freeSectorsBefore(const Partition& p) const;
     qint64 freeSectorsAfter(const Partition& p) const;
+    qint64 freeSectors() const;
 
     bool hasExtended() const;
     Partition* extended() const;
@@ -149,12 +153,19 @@ public:
         return m_LastUsable;
     }
 
+    void setFirstUsableSector(qint64 s) {
+        m_FirstUsable = s;
+    }
+    void setLastUsableSector(qint64 s) {
+        m_LastUsable = s;
+    }
+
     void updateUnallocated(const Device& d);
     void insertUnallocated(const Device& d, PartitionNode* p, qint64 start) const;
 
     bool isSectorBased(const Device& d) const;
 
-    static QList<Flag> flagList();
+    static const QList<Flag> flagList();
     static QString flagName(Flag f);
     static QStringList flagNames(Flags f);
 
@@ -176,12 +187,6 @@ protected:
     void setMaxPrimaries(qint32 n) {
         m_MaxPrimaries = n;
     }
-    void setFirstUsableSector(qint64 s) {
-        m_FirstUsable = s;
-    }
-    void setLastUsableSector(qint64 s) {
-        m_LastUsable = s;
-    }
 
 private:
     Partitions m_Children;
