@@ -45,8 +45,9 @@ public:
 
     qint64 readUsedCapacity(const QString& deviceNode) const override;
     bool check(Report& report, const QString& deviceNode) const override;
-    bool create(Report& report, const QString& deviceNode) const override;
+    bool create(Report& report, const QString& deviceNode) override;
     bool resize(Report& report, const QString& deviceNode, qint64 length) const override;
+    bool resizeOnline(Report& report, const QString& deviceNode, const QString& mountPoint, qint64 length) const override;
     bool writeLabel(Report& report, const QString& deviceNode, const QString& newLabel) override;
     bool updateUUID(Report& report, const QString& deviceNode) const override;
 
@@ -62,9 +63,15 @@ public:
     CommandSupportType supportGrow() const override {
         return m_Grow;
     }
+    CommandSupportType supportGrowOnline() const override {
+        return m_Grow;
+    }
     CommandSupportType supportShrink() const override {
         return m_Shrink;
     }
+    CommandSupportType supportShrinkOnline() const override {
+        return m_Shrink;
+    }
     CommandSupportType supportMove() const override {
         return m_Move;
     }
@@ -80,6 +87,9 @@ public:
     CommandSupportType supportSetLabel() const override {
         return m_SetLabel;
     }
+    CommandSupportType supportSetLabelOnline() const override {
+        return m_SetLabel;
+    }
     CommandSupportType supportUpdateUUID() const override {
         return m_UpdateUUID;
     }
