@@ -52,8 +52,7 @@ fat16::fat16(qint64 firstsector, qint64 lastsector, qint64 sectorsused, const QS
 
 void fat16::init()
 {
-    m_Create = findExternal(QStringLiteral("mkfs.msdos")) ? cmdSupportFileSystem : cmdSupportNone;
-    m_GetUsed = m_Check = findExternal(QStringLiteral("fsck.msdos"), {}, 2) ? cmdSupportFileSystem : cmdSupportNone;
+    m_Create = m_GetUsed = m_Check = findExternal(QStringLiteral("mkfs.fat"), {}, 1) ? cmdSupportFileSystem : cmdSupportNone;
     m_GetLabel = cmdSupportCore;
     m_SetLabel = findExternal(QStringLiteral("fatlabel")) ? cmdSupportFileSystem : cmdSupportNone;
     m_Move = cmdSupportCore;
@@ -104,9 +103,10 @@ qint64 fat16::maxLabelLength() const
 
 qint64 fat16::readUsedCapacity(const QString& deviceNode) const
 {
-    ExternalCommand cmd(QStringLiteral("fsck.msdos"), { QStringLiteral("-n"), QStringLiteral("-v"), deviceNode });
+    ExternalCommand cmd(QStringLiteral("fsck.fat"), { QStringLiteral("-n"), QStringLiteral("-v"), deviceNode });
 
-    if (cmd.run(-1) && cmd.exitCode() == 0) {
+    // Exit code 1 is returned when FAT dirty bit is set
+    if (cmd.run(-1) && (cmd.exitCode() == 0 || cmd.exitCode() == 1)) {
         qint64 usedClusters = -1;
         QRegularExpression re(QStringLiteral("files, (\\d+)/\\d+ "));
         QRegularExpressionMatch reClusters = re.match(cmd.output());
@@ -131,7 +131,7 @@ qint64 fat16::readUsedCapacity(const QString& deviceNode) const
 
 bool fat16::writeLabel(Report& report, const QString& deviceNode, const QString& newLabel)
 {
-    report.line() << xi18nc("@info/plain", "Setting label for partition <filename>%1</filename> to %2", deviceNode, newLabel);
+    report.line() << xi18nc("@info:progress", "Setting label for partition <filename>%1</filename> to %2", deviceNode, newLabel);
 
     ExternalCommand cmd(report, QStringLiteral("fatlabel"), { deviceNode, newLabel });
     return cmd.run(-1) && cmd.exitCode() == 0;
@@ -143,9 +143,9 @@ bool fat16::check(Report& report, const QString& deviceNode) const
     return cmd.run(-1) && cmd.exitCode() == 0;
 }
 
-bool fat16::create(Report& report, const QString& deviceNode) const
+bool fat16::create(Report& report, const QString& deviceNode)
 {
-    ExternalCommand cmd(report, QStringLiteral("mkfs.msdos"), { QStringLiteral("-F16"), QStringLiteral("-I"), QStringLiteral("-v"), deviceNode });
+    ExternalCommand cmd(report, QStringLiteral("mkfs.fat"), { QStringLiteral("-F16"), QStringLiteral("-I"), QStringLiteral("-v"), deviceNode });
     return cmd.run(-1) && cmd.exitCode() == 0;
 }
 
@@ -154,8 +154,10 @@ bool fat16::updateUUID(Report& report, const QString& deviceNode) const
     qint32 t = time(nullptr);
 
     char uuid[4];
-    for (quint32 i = 0; i < sizeof(uuid); i++, t >>= 8)
-        uuid[i] = t & 0xff;
+    for (auto &u : uuid) {
+        u = t & 0xff;
+        t >>= 8;
+    }
 
     ExternalCommand cmd(report, QStringLiteral("dd"), { QStringLiteral("of=") + deviceNode , QStringLiteral("bs=1"), QStringLiteral("count=4"), QStringLiteral("seek=39") });
 
