@@ -1,5 +1,6 @@
 /*************************************************************************
  *  Copyright (C) 2012 by Volker Lanz <vl@fidra.de>                      *
+ *  Copyright (C) 2015 by Teo Mrnjavac <teo@kde.org>                     *
  *  Copyright (C) 2016 by Andrius Štikonas <andrius@stikonas.eu>         *
  *                                                                       *
  *  This program is free software; you can redistribute it and/or        *
@@ -19,13 +20,13 @@
 #if !defined(FILESYSTEM__H)
 
 #define FILESYSTEM__H
-#include "../util/libpartitionmanagerexport.h"
+#include "util/libpartitionmanagerexport.h"
 
-#include <qglobal.h>
 #include <QColor>
+#include <QList>
 #include <QStringList>
 #include <QString>
-#include <QList>
+#include <QtGlobal>
 #include <QUrl>
 
 #include <array>
@@ -106,10 +107,13 @@ public:
     virtual ~FileSystem() {}
 
 public:
+    virtual void init() {};
+    virtual void scan(const QString& deviceNode);
     virtual qint64 readUsedCapacity(const QString& deviceNode) const;
     virtual QString readLabel(const QString& deviceNode) const;
-    virtual bool create(Report& report, const QString& deviceNode) const;
+    virtual bool create(Report& report, const QString& deviceNode);
     virtual bool resize(Report& report, const QString& deviceNode, qint64 newLength) const;
+    virtual bool resizeOnline(Report& report, const QString& deviceNode, const QString& mountPoint, qint64 newLength) const;
     virtual bool move(Report& report, const QString& deviceNode, qint64 newStartSector) const;
     virtual bool writeLabel(Report& report, const QString& deviceNode, const QString& newLabel);
     virtual bool copy(Report& report, const QString& targetDeviceNode, const QString& sourceDeviceNode) const;
@@ -132,15 +136,24 @@ public:
     virtual CommandSupportType supportGrow() const {
         return cmdSupportNone;    /**< @return CommandSupportType for growing */
     }
+    virtual CommandSupportType supportGrowOnline() const {
+        return cmdSupportNone;    /**< @return CommandSupportType for online growing */
+    }
     virtual CommandSupportType supportShrink() const {
         return cmdSupportNone;    /**< @return CommandSupportType for shrinking */
     }
+    virtual CommandSupportType supportShrinkOnline() const {
+        return cmdSupportNone;    /**< @return CommandSupportType for shrinking */
+    }
     virtual CommandSupportType supportMove() const {
         return cmdSupportNone;    /**< @return CommandSupportType for moving */
     }
     virtual CommandSupportType supportCheck() const {
         return cmdSupportNone;    /**< @return CommandSupportType for checking */
     }
+    virtual CommandSupportType supportCheckOnline() const {
+        return cmdSupportNone;    /**< @return CommandSupportType for checking */
+    }
     virtual CommandSupportType supportCopy() const {
         return cmdSupportNone;    /**< @return CommandSupportType for copying */
     }
@@ -148,7 +161,10 @@ public:
         return cmdSupportNone;    /**< @return CommandSupportType for backing up */
     }
     virtual CommandSupportType supportSetLabel() const {
-        return cmdSupportNone;    /**< @return CommandSupportType for setting label*/
+        return cmdSupportNone;    /**< @return CommandSupportType for setting label */
+    }
+    virtual CommandSupportType supportSetLabelOnline() const {
+        return cmdSupportNone;    /**< @return CommandSupportType for setting label of mounted file systems */
     }
     virtual CommandSupportType supportUpdateUUID() const {
         return cmdSupportNone;    /**< @return CommandSupportType for updating the UUID */
@@ -172,19 +188,21 @@ public:
     static QString nameForType(FileSystem::Type t);
     static QList<FileSystem::Type> types();
     static FileSystem::Type typeForName(const QString& s);
+    static FileSystem::Type detectFileSystem(const QString& partitionPath);
+    static QString detectMountPoint(FileSystem* fs, const QString& partitionPath);
+    static bool detectMountStatus(FileSystem* fs, const QString& partitionPath);
 
-    virtual bool canMount(const QString&) const {
-        return false;    /**< @return true if this FileSystem can be mounted */
-    }
+    /**< @return true if this FileSystem can be mounted */
+    virtual bool canMount(const QString& deviceNode, const QString& mountPoint) const;
     virtual bool canUnmount(const QString&) const {
-        return false;    /**< @return true if this FileSystem can be unmounted */
+        return true;    /**< @return true if this FileSystem can be unmounted */
     }
 
     virtual QString mountTitle() const;
     virtual QString unmountTitle() const;
 
-    virtual bool mount(const QString& mountPoint);
-    virtual bool unmount(const QString& mountPoint);
+    virtual bool mount(Report& report, const QString& deviceNode, const QString& mountPoint);
+    virtual bool unmount(Report& report, const QString& deviceNode);
 
     qint64 firstSector() const {
         return m_FirstSector;    /**< @return the FileSystem's first sector */
