@@ -18,11 +18,13 @@
  *************************************************************************/
 
 #include "fs/luks.h"
+#include "fs/lvm2_pv.h"
 
 #include "fs/filesystemfactory.h"
 
-#include "util/capacity.h"
 #include "util/externalcommand.h"
+#include "util/capacity.h"
+#include "util/helpers.h"
 #include "util/report.h"
 
 #include <QDebug>
@@ -58,6 +60,7 @@ luks::luks(qint64 firstsector,
     : FileSystem(firstsector, lastsector, sectorsused, label, FileSystem::Luks)
     , m_innerFs(nullptr)
     , m_isCryptOpen(false)
+    , m_cryptsetupFound(m_Create != cmdSupportNone)
     , m_isMounted(false)
 {
 }
@@ -69,25 +72,32 @@ luks::~luks()
 
 void luks::init()
 {
-    m_Create = findExternal(QStringLiteral("cryptsetup")) ? cmdSupportFileSystem : cmdSupportNone;
+    CommandSupportType cryptsetupFound = findExternal(QStringLiteral("cryptsetup")) ? cmdSupportFileSystem : cmdSupportNone;
+
+    m_Create     = cryptsetupFound;
+    m_UpdateUUID = cryptsetupFound;
+    m_GetUUID    = cryptsetupFound;
+    m_Grow       = cryptsetupFound;
+    m_Shrink     = cryptsetupFound;
+
     m_SetLabel = cmdSupportNone;
     m_GetLabel = cmdSupportFileSystem;
-    m_UpdateUUID = findExternal(QStringLiteral("cryptsetup")) ? cmdSupportFileSystem : cmdSupportNone;
-    m_Grow = findExternal(QStringLiteral("cryptsetup")) ? cmdSupportFileSystem : cmdSupportNone;
-    m_Shrink = m_Grow;
     m_Check = cmdSupportCore;
     m_Copy = cmdSupportCore;
     m_Move = cmdSupportCore;
     m_Backup = cmdSupportCore;
     m_GetUsed = cmdSupportNone; // libparted does not support LUKS, we do this as a special case
-    m_GetUUID = findExternal(QStringLiteral("cryptsetup")) ? cmdSupportFileSystem : cmdSupportNone;
+}
+
+void luks::scan(const QString& deviceNode)
+{
+    getMapperName(deviceNode);
+    getLuksInfo(deviceNode);
 }
 
 bool luks::supportToolFound() const
 {
-    m_cryptsetupFound = findExternal(QStringLiteral("cryptsetup")) ? cmdSupportFileSystem : cmdSupportNone;
-    return m_cryptsetupFound &&
-        ((m_isCryptOpen && m_innerFs) ? m_innerFs->supportToolFound() : true);
+    return m_cryptsetupFound && ((m_isCryptOpen && m_innerFs) ? m_innerFs->supportToolFound() : true);
 }
 
 FileSystem::SupportTool luks::supportToolName() const
@@ -98,7 +108,7 @@ FileSystem::SupportTool luks::supportToolName() const
                        QUrl(QStringLiteral("https://code.google.com/p/cryptsetup/")));
 }
 
-bool luks::create(Report& report, const QString& deviceNode) const
+bool luks::create(Report& report, const QString& deviceNode)
 {
     Q_ASSERT(m_innerFs);
     Q_ASSERT(!m_passphrase.isEmpty());
@@ -107,6 +117,7 @@ bool luks::create(Report& report, const QString& deviceNode) const
                               { QStringLiteral("-s"),
                                 QStringLiteral("512"),
                                 QStringLiteral("--batch-mode"),
+                                QStringLiteral("--force-password"),
                                 QStringLiteral("luksFormat"),
                                 deviceNode });
     if (!( createCmd.start(-1) &&
@@ -124,38 +135,35 @@ bool luks::create(Report& report, const QString& deviceNode) const
     if (!( openCmd.start(-1) &&  openCmd.write(m_passphrase.toUtf8() + '\n') == m_passphrase.toUtf8().length() + 1 && openCmd.waitFor()))
         return false;
 
-    QString mapperNode = mapperName(deviceNode);
-    if (mapperNode.isEmpty())
+    scan(deviceNode);
+
+    if (mapperName().isEmpty())
         return false;
 
-    if (!m_innerFs->create(report, mapperNode))
+    if (!m_innerFs->create(report, mapperName()))
         return false;
 
-    m_isCryptOpen = (m_innerFs != nullptr);
-
-    if (m_isCryptOpen)
-        return true;
-    return false;
+    return true;
 }
 
 QString luks::mountTitle() const
 {
-    return i18nc("@title:menu", "Mount");
+    return xi18nc("@title:menu", "Mount");
 }
 
 QString luks::unmountTitle() const
 {
-    return i18nc("@title:menu", "Unmount");
+    return xi18nc("@title:menu", "Unmount");
 }
 
 QString luks::cryptOpenTitle() const
 {
-    return i18nc("@title:menu", "Decrypt");
+    return xi18nc("@title:menu", "Decrypt");
 }
 
 QString luks::cryptCloseTitle() const
 {
-    return i18nc("@title:menu", "Deactivate");
+    return xi18nc("@title:menu", "Deactivate");
 }
 
 void luks::setPassphrase(const QString& passphrase)
@@ -168,20 +176,20 @@ QString luks::passphrase() const
     return m_passphrase;
 }
 
-bool luks::canMount(const QString& deviceNode, const QString& mountPoint) const
+bool luks::canMount(const QString&, const QString& mountPoint) const
 {
     return m_isCryptOpen &&
            !m_isMounted &&
            m_innerFs &&
-           m_innerFs->canMount(mapperName(deviceNode), mountPoint);
+           m_innerFs->canMount(mapperName(), mountPoint);
 }
 
-bool luks::canUnmount(const QString& deviceNode) const
+bool luks::canUnmount(const QString&) const
 {
     return m_isCryptOpen &&
            m_isMounted &&
            m_innerFs &&
-           m_innerFs->canUnmount(mapperName(deviceNode));
+           m_innerFs->canUnmount(mapperName());
 }
 
 bool luks::isMounted() const
@@ -218,7 +226,7 @@ bool luks::cryptOpen(QWidget* parent, const QString& deviceNode)
 {
     if (m_isCryptOpen)
     {
-        if (!mapperName(deviceNode).isEmpty())
+        if (!mapperName().isEmpty())
         {
             qWarning() << "LUKS device" << deviceNode
                        << "already decrypted."
@@ -249,29 +257,30 @@ bool luks::cryptOpen(QWidget* parent, const QString& deviceNode)
     if (!( openCmd.start(-1) &&
                     openCmd.write(passphrase.toUtf8() + '\n') == passphrase.toUtf8().length() + 1 &&
                     openCmd.waitFor() && openCmd.exitCode() == 0) )
-    {
         return false;
-    }
 
-    if (m_innerFs)
-    {
+    if (m_innerFs) {
         delete m_innerFs;
         m_innerFs = nullptr;
     }
 
-    QString mapperNode = mapperName(deviceNode);
-    if (mapperNode.isEmpty())
+    scan(deviceNode);
+
+    if (mapperName().isEmpty())
         return false;
 
-    loadInnerFileSystem(deviceNode, mapperNode);
+    loadInnerFileSystem(mapperName());
     m_isCryptOpen = (m_innerFs != nullptr);
 
-    if (m_isCryptOpen)
-    {
-        m_passphrase = passphrase;
-        return true;
-    }
-    return false;
+    if (!m_isCryptOpen)
+        return false;
+
+    for (auto &p : LVM::pvList) // FIXME: qAsConst
+        if (p.isLuks() && p.partition()->deviceNode() == deviceNode && p.partition()->fileSystem().type() == FileSystem::Lvm2_PV)
+            p.setLuks(false);
+
+    m_passphrase = passphrase;
+    return true;
 }
 
 bool luks::cryptClose(const QString& deviceNode)
@@ -291,7 +300,7 @@ bool luks::cryptClose(const QString& deviceNode)
     }
 
     ExternalCommand cmd(QStringLiteral("cryptsetup"),
-                        { QStringLiteral("close"), mapperName(deviceNode) });
+                        { QStringLiteral("close"), mapperName() });
     if (!(cmd.run(-1) && cmd.exitCode() == 0))
         return false;
 
@@ -305,12 +314,14 @@ bool luks::cryptClose(const QString& deviceNode)
 
     m_isCryptOpen = (m_innerFs != nullptr);
 
-    if (!m_isCryptOpen)
-        return true;
-    return false;
+    for (auto &p : LVM::pvList) // FIXME: qAsConst
+        if (!p.isLuks() && p.partition()->deviceNode() == deviceNode)
+            p.setLuks(true);
+
+    return true;
 }
 
-void luks::loadInnerFileSystem(const QString& deviceNode, const QString& mapperNode)
+void luks::loadInnerFileSystem(const QString& mapperNode)
 {
     Q_ASSERT(!m_innerFs);
     FileSystem::Type innerFsType = detectFileSystem(mapperNode);
@@ -319,7 +330,8 @@ void luks::loadInnerFileSystem(const QString& deviceNode, const QString& mapperN
     setLabel(m_innerFs->readLabel(mapperNode));
     setUUID(m_innerFs->readUUID(mapperNode));
     if (m_innerFs->supportGetUsed() == FileSystem::cmdSupportFileSystem)
-        setSectorsUsed(m_innerFs->readUsedCapacity(mapperNode)/m_logicalSectorSize + getPayloadOffset(deviceNode));
+        setSectorsUsed((m_innerFs->readUsedCapacity(mapperNode) + payloadOffset()) / m_logicalSectorSize );
+    m_innerFs->scan(mapperNode);
 }
 
 void luks::createInnerFileSystem(FileSystem::Type type)
@@ -328,15 +340,14 @@ void luks::createInnerFileSystem(FileSystem::Type type)
     m_innerFs = FileSystemFactory::cloneWithNewType(type, *this);
 }
 
-bool luks::check(Report& report, const QString& deviceNode) const
+bool luks::check(Report& report, const QString&) const
 {
     Q_ASSERT(m_innerFs);
 
-    QString mapperNode = mapperName(deviceNode);
-    if (mapperNode.isEmpty())
+    if (mapperName().isEmpty())
         return false;
 
-    return m_innerFs->check(report, mapperNode);
+    return m_innerFs->check(report, mapperName());
 }
 
 qint64 luks::readUsedCapacity(const QString& deviceNode) const
@@ -366,19 +377,18 @@ bool luks::mount(Report& report, const QString& deviceNode, const QString& mount
 
     Q_ASSERT(m_innerFs);
 
-    QString mapperNode = mapperName(deviceNode);
-    if (mapperNode.isEmpty())
+    if (mapperName().isEmpty())
         return false;
 
-    if (m_innerFs->canMount(mapperNode, mountPoint))
+    if (m_innerFs->canMount(mapperName(), mountPoint))
     {
-        if (m_innerFs->mount(report, mapperNode, mountPoint))
+        if (m_innerFs->mount(report, mapperName(), mountPoint))
         {
             m_isMounted = true;
 
             const KDiskFreeSpaceInfo freeSpaceInfo = KDiskFreeSpaceInfo::freeSpaceInfo(mountPoint);
             if (freeSpaceInfo.isValid() && mountPoint != QString())
-                setSectorsUsed(freeSpaceInfo.used() / m_logicalSectorSize + getPayloadOffset(deviceNode));
+                setSectorsUsed((freeSpaceInfo.used() + payloadOffset()) / m_logicalSectorSize);
 
             return true;
         }
@@ -387,7 +397,7 @@ bool luks::mount(Report& report, const QString& deviceNode, const QString& mount
         ExternalCommand mountCmd(
                 report,
                 QStringLiteral("mount"),
-                { QStringLiteral("--verbose"), mapperNode, mountPoint });
+                { QStringLiteral("--verbose"), mapperName(), mountPoint });
         if (mountCmd.run() && mountCmd.exitCode() == 0)
         {
             m_isMounted = true;
@@ -415,13 +425,12 @@ bool luks::unmount(Report& report, const QString& deviceNode)
 
     Q_ASSERT(m_innerFs);
 
-    QString mapperNode = mapperName(deviceNode);
-    if (mapperNode.isEmpty())
+    if (mapperName().isEmpty())
         return false;
 
-    if (m_innerFs->canUnmount(mapperNode))
+    if (m_innerFs->canUnmount(mapperName()))
     {
-        if (m_innerFs->unmount(report, mapperNode))
+        if (m_innerFs->unmount(report, mapperName()))
         {
             m_isMounted = false;
             return true;
@@ -430,7 +439,7 @@ bool luks::unmount(Report& report, const QString& deviceNode)
     else {
         ExternalCommand unmountCmd( report,
                 QStringLiteral("umount"),
-                { QStringLiteral("--verbose"), QStringLiteral("--all-targets"), mapperNode });
+                { QStringLiteral("--verbose"), QStringLiteral("--all-targets"), mapperName() });
         if (unmountCmd.run() && unmountCmd.exitCode() == 0)
         {
             m_isMounted = false;
@@ -452,55 +461,52 @@ QString luks::suggestedMapperName(const QString& deviceNode) const
     return QStringLiteral("luks-") + readOuterUUID(deviceNode);
 }
 
-QString luks::readLabel(const QString& deviceNode) const
+QString luks::readLabel(const QString&) const
 {
     if (m_isCryptOpen && m_innerFs)
-        return m_innerFs->readLabel(mapperName(deviceNode));
+        return m_innerFs->readLabel(mapperName());
     return QString();
 }
 
-bool luks::writeLabel(Report& report, const QString& deviceNode, const QString& newLabel)
+bool luks::writeLabel(Report& report, const QString&, const QString& newLabel)
 {
     Q_ASSERT(m_innerFs);
-    return m_innerFs->writeLabel(report, mapperName(deviceNode), newLabel);
+    return m_innerFs->writeLabel(report, mapperName(), newLabel);
 }
 
 bool luks::resize(Report& report, const QString& deviceNode, qint64 newLength) const
 {
     Q_ASSERT(m_innerFs);
 
-    QString mapperNode = mapperName(deviceNode);
-    if (mapperNode.isEmpty())
+    if (mapperName().isEmpty())
         return false;
 
-    qint64 payloadLength = newLength - getPayloadOffset(deviceNode) * m_logicalSectorSize;
+    qint64 payloadLength = newLength - payloadOffset();
     if ( newLength - length() * m_logicalSectorSize > 0 )
     {
-        ExternalCommand cryptResizeCmd(report, QStringLiteral("cryptsetup"), { QStringLiteral("resize"), mapperNode });
-        report.line() << xi18nc("@info/plain", "Resizing LUKS crypt on partition <filename>%1</filename>.", deviceNode);
+        ExternalCommand cryptResizeCmd(report, QStringLiteral("cryptsetup"), { QStringLiteral("resize"), mapperName() });
+        report.line() << xi18nc("@info:progress", "Resizing LUKS crypt on partition <filename>%1</filename>.", deviceNode);
 
         if (cryptResizeCmd.run(-1) && cryptResizeCmd.exitCode() == 0)
-        {
-            return m_innerFs->resize(report, mapperNode, newLength - getPayloadOffset(deviceNode) * m_logicalSectorSize);
-        }
+            return m_innerFs->resize(report, mapperName(), payloadLength);
     }
-    else if (m_innerFs->resize(report, mapperNode, payloadLength))
+    else if (m_innerFs->resize(report, mapperName(), payloadLength))
     {
-        ExternalCommand cryptResizeCmd(report, QStringLiteral("cryptsetup"), { QStringLiteral("--size"), QString::number(payloadLength / m_logicalSectorSize), QStringLiteral("resize"), mapperNode });
-        report.line() << xi18nc("@info/plain", "Resizing LUKS crypt on partition <filename>%1</filename>.", deviceNode);
+        ExternalCommand cryptResizeCmd(report, QStringLiteral("cryptsetup"),
+                {  QStringLiteral("--size"), QString::number(payloadLength / m_logicalSectorSize), // LUKS assumes 512 bytes sector
+                   QStringLiteral("resize"), mapperName() });
+        report.line() << xi18nc("@info:progress", "Resizing LUKS crypt on partition <filename>%1</filename>.", deviceNode);
         if (cryptResizeCmd.run(-1) && cryptResizeCmd.exitCode() == 0)
-        {
             return true;
-        }
     }
-    report.line() << xi18nc("@info/plain", "Resizing encrypted file system on partition <filename>%1</filename> failed.", deviceNode);
+    report.line() << xi18nc("@info:progress", "Resizing encrypted file system on partition <filename>%1</filename> failed.", deviceNode);
     return false;
 }
 
 QString luks::readUUID(const QString& deviceNode) const
 {
     if (m_isCryptOpen && m_innerFs)
-        return m_innerFs->readUUID(mapperName(deviceNode));
+        return m_innerFs->readUUID(mapperName());
     return readOuterUUID(deviceNode);
 }
 
@@ -527,7 +533,7 @@ bool luks::updateUUID(Report& report, const QString& deviceNode) const
     return cmd.run(-1) && cmd.exitCode() == 0;
 }
 
-QString luks::mapperName(const QString& deviceNode)
+void luks::getMapperName(const QString& deviceNode)
 {
     ExternalCommand cmd(QStringLiteral("lsblk"),
                         { QStringLiteral("--list"),
@@ -539,94 +545,96 @@ QString luks::mapperName(const QString& deviceNode)
         QStringList output=cmd.output().split(QStringLiteral("\n"));
         output.removeFirst();
         if (!output.first().isEmpty())
-            return QStringLiteral("/dev/mapper/") + output.first();
+            m_MapperName = QStringLiteral("/dev/mapper/") + output.first();
     }
-    return QString();
+    else
+        m_MapperName = QString();
 }
 
-QString luks::getCipherName(const QString& deviceNode) const
+void luks::getLuksInfo(const QString& deviceNode)
 {
     ExternalCommand cmd(QStringLiteral("cryptsetup"),
                         { QStringLiteral("luksDump"), deviceNode });
     if (cmd.run(-1) && cmd.exitCode() == 0) {
         QRegularExpression re(QStringLiteral("Cipher name:\\s+(\\w+)"));
-        QRegularExpressionMatch reCipherName = re.match(cmd.output());
-        if (reCipherName.hasMatch())
-            return reCipherName.captured(1);
-    }
-    return QStringLiteral("---");
-}
+        QRegularExpressionMatch rem = re.match(cmd.output());
+        if (rem.hasMatch())
+            m_CipherName = rem.captured(1);
+        else
+            m_CipherName = QLatin1String("---");
 
-QString luks::getCipherMode(const QString& deviceNode) const
-{
-    ExternalCommand cmd(QStringLiteral("cryptsetup"),
-                        { QStringLiteral("luksDump"), deviceNode });
-    if (cmd.run(-1) && cmd.exitCode() == 0) {
-        QRegularExpression re(QStringLiteral("Cipher mode:\\s+(\\w+)"));
-        QRegularExpressionMatch reCipherMode = re.match(cmd.output());
-        if (reCipherMode.hasMatch())
-            return reCipherMode.captured(1);
-    }
-    return QStringLiteral("---");
-}
+        re.setPattern(QStringLiteral("Cipher mode:\\s+(\\w+)"));
+        rem = re.match(cmd.output());
+        if (rem.hasMatch())
+            m_CipherMode = rem.captured(1);
+        else
+            m_CipherMode = QLatin1String("---");
 
-QString luks::getHashName(const QString& deviceNode) const
-{
-    ExternalCommand cmd(QStringLiteral("cryptsetup"),
-                        { QStringLiteral("luksDump"), deviceNode });
-    if (cmd.run(-1) && cmd.exitCode() == 0) {
-        QRegularExpression re(QStringLiteral("Hash spec:\\s+(\\w+)"));
-        QRegularExpressionMatch reHash = re.match(cmd.output());
-        if (reHash.hasMatch())
-            return reHash.captured(1);
-    }
-    return QStringLiteral("---");
-}
+        re.setPattern(QStringLiteral("Hash spec:\\s+(\\w+)"));
+        rem = re.match(cmd.output());
+        if (rem.hasMatch())
+            m_HashName = rem.captured(1);
+        else
+            m_HashName = QLatin1String("---");
 
-qint64 luks::getKeySize(const QString& deviceNode) const
-{
-    ExternalCommand cmd(QStringLiteral("cryptsetup"),
-                        { QStringLiteral("luksDump"), deviceNode });
-    if (cmd.run(-1) && cmd.exitCode() == 0) {
-        QRegularExpression re(QStringLiteral("MK bits:\\s+(\\d+)"));
-        QRegularExpressionMatch reKeySize = re.match(cmd.output());
-        if (reKeySize.hasMatch())
-            return reKeySize.captured(1).toLongLong();
-    }
-    return -1;
-}
+        re.setPattern(QStringLiteral("MK bits:\\s+(\\d+)"));
+        rem = re.match(cmd.output());
+        if (rem.hasMatch())
+            m_KeySize = rem.captured(1).toLongLong();
+        else
+            m_KeySize = -1;
+
+        re.setPattern(QStringLiteral("Payload offset:\\s+(\\d+)"));
+        rem = re.match(cmd.output());
+        if (rem.hasMatch())
+            m_PayloadOffset = rem.captured(1).toLongLong() * 512; // assuming LUKS sector size is 512;
+        else
+            m_PayloadOffset = -1;
 
-qint64 luks::getPayloadOffset(const QString& deviceNode) const
-{
-    ExternalCommand cmd(QStringLiteral("cryptsetup"),
-                        { QStringLiteral("luksDump"), deviceNode });
-    if (cmd.run(-1) && cmd.exitCode() == 0) {
-        QRegularExpression re(QStringLiteral("Payload offset:\\s+(\\d+)"));
-        QRegularExpressionMatch rePayloadOffset = re.match(cmd.output());
-        if (rePayloadOffset.hasMatch())
-            return rePayloadOffset.captured(1).toLongLong();
     }
-    return -1;
+    else {
+        m_CipherName = QLatin1String("---");
+        m_CipherMode = QLatin1String("---");
+        m_HashName = QLatin1String("---");
+        m_KeySize = -1;
+        m_PayloadOffset = -1;
+    }
 }
 
 bool luks::canEncryptType(FileSystem::Type type)
 {
     switch (type)
     {
-    case Ext2:
-    case Ext3:
-    case Ext4:
-    case LinuxSwap:
-    case ReiserFS:
-    case Reiser4:
-    case Xfs:
-    case Jfs:
-    case Btrfs:
-    case Zfs:
-    case Lvm2_PV:
-        return true;
-    default:
-        return false;
+        case Btrfs:
+        case F2fs:
+        case Ext2:
+        case Ext3:
+        case Ext4:
+        case Jfs:
+        case LinuxSwap:
+        case Lvm2_PV:
+        case Nilfs2:
+        case ReiserFS:
+        case Reiser4:
+        case Xfs:
+        case Zfs:
+            return true;
+        default:
+            return false;
+    }
+}
+
+void luks::initLUKS(FileSystem* fs)
+{
+    if (fs->type() == FileSystem::Luks) {
+        FS::luks* luksFS = static_cast<FS::luks*>(fs);
+        QString mapperNode = luksFS->mapperName();
+        bool isCryptOpen = !mapperNode.isEmpty();
+        luksFS->setCryptOpen(isCryptOpen);
+        if (isCryptOpen) {
+            luksFS->loadInnerFileSystem(mapperNode);
+            luksFS->setMounted(detectMountStatus(luksFS->innerFS(), mapperNode));
+        }
     }
 }
 
