@@ -143,7 +143,7 @@ bool xfs::check(Report& report, const QString& deviceNode) const
     return cmd.run(-1) && cmd.exitCode() == 0;
 }
 
-bool xfs::create(Report& report, const QString& deviceNode) const
+bool xfs::create(Report& report, const QString& deviceNode)
 {
     ExternalCommand cmd(report, QStringLiteral("mkfs.xfs"), { QStringLiteral("-f"), deviceNode });
     return cmd.run(-1) && cmd.exitCode() == 0;
@@ -166,7 +166,7 @@ bool xfs::resize(Report& report, const QString& deviceNode, qint64) const
 {
     QTemporaryDir tempDir;
     if (!tempDir.isValid()) {
-        report.line() << xi18nc("@info/plain", "Resizing XFS file system on partition <filename>%1</filename> failed: Could not create temp dir.", deviceNode);
+        report.line() << xi18nc("@info:progress", "Resizing XFS file system on partition <filename>%1</filename> failed: Could not create temp dir.", deviceNode);
         return false;
     }
 
@@ -177,18 +177,29 @@ bool xfs::resize(Report& report, const QString& deviceNode, qint64) const
     if (mountCmd.run(-1)) {
         ExternalCommand resizeCmd(report, QStringLiteral("xfs_growfs"), { tempDir.path() });
 
-        if (resizeCmd.run(-1))
+        if (resizeCmd.run(-1) && resizeCmd.exitCode() == 0)
             rval = true;
         else
-            report.line() << xi18nc("@info/plain", "Resizing XFS file system on partition <filename>%1</filename> failed: xfs_growfs failed.", deviceNode);
+            report.line() << xi18nc("@info:progress", "Resizing XFS file system on partition <filename>%1</filename> failed: xfs_growfs failed.", deviceNode);
 
         ExternalCommand unmountCmd(report, QStringLiteral("umount"), { tempDir.path() });
 
         if (!unmountCmd.run(-1))
-            report.line() << xi18nc("@info/plain", "Warning: Resizing XFS file system on partition <filename>%1</filename>: Unmount failed.", deviceNode);
+            report.line() << xi18nc("@info:progress", "<warning>Resizing XFS file system on partition <filename>%1</filename> failed: Unmount failed.</warning>", deviceNode);
     } else
-        report.line() << xi18nc("@info/plain", "Resizing XFS file system on partition <filename>%1</filename> failed: Initial mount failed.", deviceNode);
+        report.line() << xi18nc("@info:progress", "Resizing XFS file system on partition <filename>%1</filename> failed: Initial mount failed.", deviceNode);
 
     return rval;
 }
+
+bool xfs::resizeOnline(Report& report, const QString& deviceNode, const QString& mountPoint, qint64) const
+{
+    ExternalCommand resizeCmd(report, QStringLiteral("xfs_growfs"), { mountPoint });
+
+    if (resizeCmd.run(-1) && resizeCmd.exitCode() == 0)
+        return true;
+
+    report.line() << xi18nc("@info:progress", "Resizing XFS file system on partition <filename>%1</filename> failed: xfs_growfs failed.", deviceNode);
+    return false;
+}
 }
