@@ -56,7 +56,7 @@ bool CopyFileSystemJob::run(Report& parent)
     Report* report = jobStarted(parent);
 
     if (targetPartition().fileSystem().length() < sourcePartition().fileSystem().length())
-        report->line() << xi18nc("@info/plain", "Cannot copy file system: File system on target partition <filename>%1</filename> is smaller than the file system on source partition <filename>%2</filename>.", targetPartition().deviceNode(), sourcePartition().deviceNode());
+        report->line() << xi18nc("@info:progress", "Cannot copy file system: File system on target partition <filename>%1</filename> is smaller than the file system on source partition <filename>%2</filename>.", targetPartition().deviceNode(), sourcePartition().deviceNode());
     else if (sourcePartition().fileSystem().supportCopy() == FileSystem::cmdSupportFileSystem)
         rval = sourcePartition().fileSystem().copy(*report, targetPartition().deviceNode(), sourcePartition().deviceNode());
     else if (sourcePartition().fileSystem().supportCopy() == FileSystem::cmdSupportCore) {
@@ -64,12 +64,12 @@ bool CopyFileSystemJob::run(Report& parent)
         CopyTargetDevice copyTarget(targetDevice(), targetPartition().fileSystem().firstSector(), targetPartition().fileSystem().lastSector());
 
         if (!copySource.open())
-            report->line() << xi18nc("@info/plain", "Could not open file system on source partition <filename>%1</filename> for copying.", sourcePartition().deviceNode());
+            report->line() << xi18nc("@info:progress", "Could not open file system on source partition <filename>%1</filename> for copying.", sourcePartition().deviceNode());
         else if (!copyTarget.open())
-            report->line() << xi18nc("@info/plain", "Could not open file system on target partition <filename>%1</filename> for copying.", targetPartition().deviceNode());
+            report->line() << xi18nc("@info:progress", "Could not open file system on target partition <filename>%1</filename> for copying.", targetPartition().deviceNode());
         else {
             rval = copyBlocks(*report, copyTarget, copySource);
-            report->line() << i18nc("@info/plain", "Closing device. This may take a while, especially on slow devices like Memory Sticks.");
+            report->line() << xi18nc("@info:progress", "Closing device. This may take a while, especially on slow devices like Memory Sticks.");
         }
     }
 
@@ -96,5 +96,5 @@ bool CopyFileSystemJob::run(Report& parent)
 
 QString CopyFileSystemJob::description() const
 {
-    return xi18nc("@info/plain", "Copy file system on partition <filename>%1</filename> to partition <filename>%2</filename>", sourcePartition().deviceNode(), targetPartition().deviceNode());
+    return xi18nc("@info:progress", "Copy file system on partition <filename>%1</filename> to partition <filename>%2</filename>", sourcePartition().deviceNode(), targetPartition().deviceNode());
 }
