@@ -22,8 +22,9 @@
 
 #include "plugins/libparted/libpartedbackend.h"
 #include "plugins/libparted/libparteddevice.h"
+#include "plugins/libparted/pedflags.h"
 
-#include "core/device.h"
+#include "core/diskdevice.h"
 #include "core/partition.h"
 #include "core/partitiontable.h"
 #include "core/partitionalignment.h"
@@ -35,8 +36,10 @@
 #include "fs/hfs.h"
 #include "fs/hfsplus.h"
 #include "fs/luks.h"
+#include "fs/lvm2_pv.h"
 
 #include "util/globallog.h"
+#include "util/externalcommand.h"
 #include "util/helpers.h"
 
 #include <blkid/blkid.h>
@@ -46,7 +49,6 @@
 #include <QStringList>
 
 #include <KLocalizedString>
-#include <KMountPoint>
 #include <KDiskFreeSpaceInfo>
 #include <KPluginFactory>
 
@@ -55,30 +57,6 @@
 
 K_PLUGIN_FACTORY_WITH_JSON(LibPartedBackendFactory, "pmlibpartedbackendplugin.json", registerPlugin<LibPartedBackend>();)
 
-static struct {
-    PedPartitionFlag pedFlag;
-    PartitionTable::Flag flag;
-} flagmap[] = {
-    { PED_PARTITION_BOOT,               PartitionTable::FlagBoot },
-    { PED_PARTITION_ROOT,               PartitionTable::FlagRoot },
-    { PED_PARTITION_SWAP,               PartitionTable::FlagSwap },
-    { PED_PARTITION_HIDDEN,             PartitionTable::FlagHidden },
-    { PED_PARTITION_RAID,               PartitionTable::FlagRaid },
-    { PED_PARTITION_LVM,                PartitionTable::FlagLvm },
-    { PED_PARTITION_LBA,                PartitionTable::FlagLba },
-    { PED_PARTITION_HPSERVICE,          PartitionTable::FlagHpService },
-    { PED_PARTITION_PALO,               PartitionTable::FlagPalo },
-    { PED_PARTITION_PREP,               PartitionTable::FlagPrep },
-    { PED_PARTITION_MSFT_RESERVED,      PartitionTable::FlagMsftReserved },
-    { PED_PARTITION_BIOS_GRUB,          PartitionTable::FlagBiosGrub },
-    { PED_PARTITION_APPLE_TV_RECOVERY,  PartitionTable::FlagAppleTvRecovery },
-    { PED_PARTITION_DIAG,               PartitionTable::FlagDiag }, // generic diagnostics flag
-    { PED_PARTITION_LEGACY_BOOT,        PartitionTable::FlagLegacyBoot },
-    { PED_PARTITION_MSFT_DATA,          PartitionTable::FlagMsftData },
-    { PED_PARTITION_IRST,               PartitionTable::FlagIrst }, // Intel Rapid Start partition
-    { PED_PARTITION_ESP,                PartitionTable::FlagEsp }   // EFI system
-};
-
 static QString s_lastPartedExceptionMessage;
 
 /** Callback to handle exceptions from libparted
@@ -86,7 +64,7 @@ static QString s_lastPartedExceptionMessage;
 */
 static PedExceptionOption pedExceptionHandler(PedException* e)
 {
-    Log(Log::error) << i18nc("@info/plain", "LibParted Exception: %1", QString::fromLocal8Bit(e->message));
+    Log(Log::error) << xi18nc("@info:status", "LibParted Exception: %1", QString::fromLocal8Bit(e->message));
     s_lastPartedExceptionMessage = QString::fromLocal8Bit(e->message);
     return PED_EXCEPTION_UNHANDLED;
 }
@@ -216,19 +194,19 @@ static qint64 readSectorsUsedLibParted(PedDisk* pedDisk, const Partition& p)
 */
 static void readSectorsUsed(PedDisk* pedDisk, const Device& d, Partition& p, const QString& mountPoint)
 {
-    Q_ASSERT(pedDisk);
+    if (mountPoint != QString()) {
+        const KDiskFreeSpaceInfo freeSpaceInfo = KDiskFreeSpaceInfo::freeSpaceInfo(mountPoint);
 
-    const KDiskFreeSpaceInfo freeSpaceInfo = KDiskFreeSpaceInfo::freeSpaceInfo(mountPoint);
-
-    if (p.isMounted() && freeSpaceInfo.isValid() && mountPoint != QString())
-        p.fileSystem().setSectorsUsed(freeSpaceInfo.used() / d.logicalSectorSize());
-    else if (p.fileSystem().supportGetUsed() == FileSystem::cmdSupportFileSystem)
-        p.fileSystem().setSectorsUsed(p.fileSystem().readUsedCapacity(p.deviceNode()) / d.logicalSectorSize());
+        if (p.isMounted() && freeSpaceInfo.isValid() && mountPoint != QString())
+            p.fileSystem().setSectorsUsed(freeSpaceInfo.used() / d.logicalSize());
+        else if (p.fileSystem().supportGetUsed() == FileSystem::cmdSupportFileSystem)
+            p.fileSystem().setSectorsUsed(p.fileSystem().readUsedCapacity(p.deviceNode()) / d.logicalSize());
+    }
 #if defined LIBPARTED_FS_RESIZE_LIBRARY_SUPPORT
     else if (p.fileSystem().supportGetUsed() == FileSystem::cmdSupportCore)
         p.fileSystem().setSectorsUsed(readSectorsUsedLibParted(pedDisk, p));
 #else
-    Q_UNUSED(pedDisk);
+        Q_UNUSED(pedDisk);
 #endif
 }
 
@@ -242,9 +220,9 @@ static PartitionTable::Flags activeFlags(PedPartition* p)
     if (p->num <= 0)
         return flags;
 
-    for (quint32 i = 0; i < sizeof(flagmap) / sizeof(flagmap[0]); i++)
-        if (ped_partition_is_flag_available(p, flagmap[i].pedFlag) && ped_partition_get_flag(p, flagmap[i].pedFlag))
-            flags |= flagmap[i].flag;
+    for (const auto &flag : flagmap)
+        if (ped_partition_is_flag_available(p, flag.pedFlag) && ped_partition_get_flag(p, flag.pedFlag))
+            flags |= flag.flag;
 
     return flags;
 }
@@ -257,13 +235,14 @@ static PartitionTable::Flags availableFlags(PedPartition* p)
     if (p->num <= 0)
         return flags;
 
-    for (quint32 i = 0; i < sizeof(flagmap) / sizeof(flagmap[0]); i++)
-        if (ped_partition_is_flag_available(p, flagmap[i].pedFlag)) {
+    for (const auto &flag : flagmap) {
+        if (ped_partition_is_flag_available(p, flag.pedFlag)) {
             // Workaround: libparted claims the hidden flag is available for extended partitions, but
             // throws an error when we try to set or clear it. So skip this combination. Also see setFlag.
-            if (p->type != PED_PARTITION_EXTENDED || flagmap[i].flag != PartitionTable::FlagHidden)
-                flags |= flagmap[i].flag;
+            if (p->type != PED_PARTITION_EXTENDED || flag.flag != PartitionTable::FlagHidden)
+                flags |= flag.flag;
         }
+    }
 
     return flags;
 }
@@ -314,9 +293,6 @@ void LibPartedBackend::scanDevicePartitions(Device& d, PedDisk* pedDisk)
 
     PedPartition* pedPartition = nullptr;
 
-    KMountPoint::List mountPoints = KMountPoint::currentMountPoints(KMountPoint::NeedRealDeviceName);
-    mountPoints.append(KMountPoint::possibleMountPoints(KMountPoint::NeedRealDeviceName));
-
     QList<Partition*> partitions;
 
     while ((pedPartition = ped_disk_next_partition(pedDisk, pedPartition))) {
@@ -327,9 +303,9 @@ void LibPartedBackend::scanDevicePartitions(Device& d, PedDisk* pedDisk)
 
         FileSystem::Type type = FileSystem::Unknown;
         char* pedPath = ped_partition_get_path(pedPartition);
-        if (pedPath)
-            type = detectFileSystem(QString::fromUtf8(pedPath));
+        const QString partitionNode = pedPath ? QString::fromUtf8(pedPath) : QString();
         free(pedPath);
+        type = detectFileSystem(partitionNode);
 
         switch (pedPartition->type) {
         case PED_PARTITION_NORMAL:
@@ -356,49 +332,24 @@ void LibPartedBackend::scanDevicePartitions(Device& d, PedDisk* pedDisk)
         if (parent == nullptr)
             parent = d.partitionTable();
 
-        pedPath = ped_partition_get_path(pedPartition);
-        const QString node = QString::fromUtf8(pedPath);
-        free(pedPath);
         FileSystem* fs = FileSystemFactory::create(type, pedPartition->geom.start, pedPartition->geom.end);
+        fs->scan(partitionNode);
+        QString mountPoint;
+        bool mounted;
 
         // libparted does not handle LUKS partitions
-        QString mountPoint;
-        bool mounted = false;
-        if (type == FileSystem::Luks) {
+        if (fs->type() == FileSystem::Luks) {
             r |= PartitionRole::Luks;
-            FS::luks* luksFs = dynamic_cast<FS::luks*>(fs);
-            QString mapperNode = FS::luks::mapperName(node);
-            bool isCryptOpen = !mapperNode.isEmpty();
-            luksFs->setCryptOpen(isCryptOpen);
-            luksFs->setLogicalSectorSize(d.logicalSectorSize());
-
-            if (isCryptOpen) {
-                luksFs->loadInnerFileSystem(node, mapperNode);
-
-                mountPoint = mountPoints.findByDevice(mapperNode) ?
-                             mountPoints.findByDevice(mapperNode)->mountPoint() :
-                             QString();
-                // We cannot use libparted to check the mounted status because
-                // we don't have a PedPartition for the mapper device, so we use lsblk
-                mounted = isMounted(mapperNode);
-                if (mounted) {
-                    const KDiskFreeSpaceInfo freeSpaceInfo = KDiskFreeSpaceInfo::freeSpaceInfo(mountPoint);
-                    if (freeSpaceInfo.isValid() && mountPoint != QString())
-                        luksFs->setSectorsUsed(freeSpaceInfo.used() / d.logicalSectorSize() + luksFs->getPayloadOffset(node));
-                }
-            } else {
-                mounted = false;
-            }
-
-            luksFs->setMounted(mounted);
+            FS::luks::initLUKS(fs);
+            QString mapperNode = static_cast<FS::luks*>(fs)->mapperName();
+            mountPoint = FileSystem::detectMountPoint(fs, mapperNode);
+            mounted    = FileSystem::detectMountStatus(fs, mapperNode);
         } else {
-            mountPoint = mountPoints.findByDevice(node) ?
-                         mountPoints.findByDevice(node)->mountPoint() :
-                         QString();
-            mounted = ped_partition_is_busy(pedPartition);
+            mountPoint = FileSystem::detectMountPoint(fs, partitionNode);
+            mounted = FileSystem::detectMountStatus(fs, partitionNode);
         }
 
-        Partition* part = new Partition(parent, d, PartitionRole(r), fs, pedPartition->geom.start, pedPartition->geom.end, node, availableFlags(pedPartition), mountPoint, mounted, activeFlags(pedPartition));
+        Partition* part = new Partition(parent, d, PartitionRole(r), fs, pedPartition->geom.start, pedPartition->geom.end, partitionNode, availableFlags(pedPartition), mountPoint, mounted, activeFlags(pedPartition));
 
         if (!part->roles().has(PartitionRole::Luks))
             readSectorsUsed(pedDisk, d, *part, mountPoint);
@@ -406,6 +357,10 @@ void LibPartedBackend::scanDevicePartitions(Device& d, PedDisk* pedDisk)
         if (fs->supportGetLabel() != FileSystem::cmdSupportNone)
             fs->setLabel(fs->readLabel(part->deviceNode()));
 
+        // GPT partitions support partition labels and partition UUIDs
+        if(d.partitionTable()->type() == PartitionTable::TableType::gpt)
+            fs->setLabel(fs->readLabel(part->deviceNode()));
+
         if (fs->supportGetUUID() != FileSystem::cmdSupportNone)
             fs->setUUID(fs->readUUID(part->deviceNode()));
 
@@ -420,26 +375,24 @@ void LibPartedBackend::scanDevicePartitions(Device& d, PedDisk* pedDisk)
 
     foreach(const Partition * part, partitions)
         PartitionAlignment::isAligned(d, *part);
-
-    ped_disk_destroy(pedDisk);
 }
 
 /** Create a Device for the given device_node and scan it for partitions.
-    @param device_node the device node (e.g. "/dev/sda")
+    @param deviceNode the device node (e.g. "/dev/sda")
     @return the created Device object. callers need to free this.
 */
-Device* LibPartedBackend::scanDevice(const QString& device_node)
+Device* LibPartedBackend::scanDevice(const QString& deviceNode)
 {
-    PedDevice* pedDevice = ped_device_get(device_node.toLocal8Bit().constData());
+    PedDevice* pedDevice = ped_device_get(deviceNode.toLocal8Bit().constData());
 
     if (pedDevice == nullptr) {
-        Log(Log::warning) << xi18nc("@info/plain", "Could not access device <filename>%1</filename>", device_node);
+        Log(Log::warning) << xi18nc("@info:status", "Could not access device <filename>%1</filename>", deviceNode);
         return nullptr;
     }
 
-    Log(Log::information) << i18nc("@info/plain", "Device found: %1", QString::fromUtf8(pedDevice->model));
+    Log(Log::information) << xi18nc("@info:status", "Device found: %1", QString::fromUtf8(pedDevice->model));
 
-    Device* d = new Device(QString::fromUtf8(pedDevice->model), QString::fromUtf8(pedDevice->path), pedDevice->bios_geom.heads, pedDevice->bios_geom.sectors, pedDevice->bios_geom.cylinders, pedDevice->sector_size);
+    DiskDevice* d = new DiskDevice(QString::fromUtf8(pedDevice->model), QString::fromUtf8(pedDevice->path), pedDevice->bios_geom.heads, pedDevice->bios_geom.sectors, pedDevice->bios_geom.cylinders, pedDevice->sector_size);
 
     PedDisk* pedDisk = ped_disk_new(pedDevice);
 
@@ -458,30 +411,36 @@ Device* LibPartedBackend::scanDevice(const QString& device_node)
 QList<Device*> LibPartedBackend::scanDevices(bool excludeReadOnly)
 {
     QList<Device*> result;
+    // linux.git/tree/Documentation/devices.txt
+    QString blockDeviceMajorNumbers = QStringLiteral(
+        "3,22,33,34,56,57,88,89,90,91,128,129,130,131,132,133,134,135," // MFM, RLL and IDE hard disk/CD-ROM interface
+        "7," // loop devices
+        "8,65,66,67,68,69,70,71," // SCSI disk devices
+        "80,81,82,83,84,85,86,87," // I2O hard disk
+        "179," // MMC block devices
+        "259" // Block Extended Major (include NVMe)
+    );
+    ExternalCommand cmd(QStringLiteral("lsblk"), {
+                          QStringLiteral("--nodeps"),
+                          QStringLiteral("--noheadings"),
+                          QStringLiteral("--output"), QString::fromLatin1("name"),
+                          QStringLiteral("--paths"),
+                          QStringLiteral("--include"), blockDeviceMajorNumbers});
+    if (cmd.run(-1) && cmd.exitCode() == 0) {
+        QStringList devices = cmd.output().split(QString::fromLatin1("\n"));
+        devices.removeLast();
+        quint32 totalDevices = devices.length();
+        for (quint32 i = 0; i < totalDevices; ++i) {
+            if (excludeReadOnly) {
+                QFile f(QStringLiteral("/sys/block/%1/ro").arg(QString(devices[i]).remove(QStringLiteral("/dev/"))));
+                if (f.open(QIODevice::ReadOnly))
+                    if (f.readLine().trimmed().toInt() == 1)
+                        continue;
+            }
 
-    ped_device_probe_all();
-    PedDevice* pedDevice = nullptr;
-    QVector<QString> path;
-    quint32 totalDevices = 0;
-    while (true) {
-        pedDevice = ped_device_get_next(pedDevice);
-        if (!pedDevice)
-            break;
-        if (pedDevice->type == PED_DEVICE_DM)
-            continue;
-        if (excludeReadOnly && (
-                pedDevice->type == PED_DEVICE_LOOP ||
-                pedDevice->read_only))
-            continue;
-
-        path.push_back(QString::fromUtf8(pedDevice->path));
-        ++totalDevices;
-    }
-    for (quint32 i = 0; i < totalDevices; ++i) {
-        emitScanProgress(path[i], i * 100 / totalDevices);
-        Device* d = scanDevice(path[i]);
-        if (d)
-            result.append(d);
+            emitScanProgress(devices[i], i * 100 / totalDevices);
+            result.append(scanDevice(devices[i]));
+        }
     }
 
     return result;
@@ -546,9 +505,9 @@ FileSystem::Type LibPartedBackend::detectFileSystem(const QString& partitionPath
     return rval;
 }
 
-CoreBackendDevice* LibPartedBackend::openDevice(const QString& device_node)
+CoreBackendDevice* LibPartedBackend::openDevice(const QString& deviceNode)
 {
-    LibPartedDevice* device = new LibPartedDevice(device_node);
+    LibPartedDevice* device = new LibPartedDevice(deviceNode);
 
     if (device == nullptr || !device->open()) {
         delete device;
@@ -558,9 +517,9 @@ CoreBackendDevice* LibPartedBackend::openDevice(const QString& device_node)
     return device;
 }
 
-CoreBackendDevice* LibPartedBackend::openDeviceExclusive(const QString& device_node)
+CoreBackendDevice* LibPartedBackend::openDeviceExclusive(const QString& deviceNode)
 {
-    LibPartedDevice* device = new LibPartedDevice(device_node);
+    LibPartedDevice* device = new LibPartedDevice(deviceNode);
 
     if (device == nullptr || !device->openExclusive()) {
         delete device;
@@ -577,9 +536,9 @@ bool LibPartedBackend::closeDevice(CoreBackendDevice* core_device)
 
 PedPartitionFlag LibPartedBackend::getPedFlag(PartitionTable::Flag flag)
 {
-    for (quint32 i = 0; i < sizeof(flagmap) / sizeof(flagmap[0]); i++)
-        if (flagmap[i].flag == flag)
-            return flagmap[i].pedFlag;
+    for (const auto &f : flagmap)
+        if (f.flag == flag)
+            return f.pedFlag;
 
     return static_cast<PedPartitionFlag>(-1);
 }
