@@ -18,8 +18,8 @@
 
 #include "util/htmlreport.h"
 
-#include "../backend/corebackend.h"
-#include "../backend/corebackendmanager.h"
+#include "backend/corebackend.h"
+#include "backend/corebackendmanager.h"
 
 #include <QApplication>
 #include <QDateTime>
@@ -29,7 +29,7 @@
 
 #include <KAboutData>
 #include <KLocalizedString>
-#include <kxmlgui_version.h>
+#include <kio_version.h>
 
 #include <sys/utsname.h>
 #include <unistd.h>
@@ -78,7 +78,7 @@ QString HtmlReport::header()
       << tableLine(i18n("Date:"), QLocale().toString(QDateTime::currentDateTime(), QLocale::ShortFormat))
       << tableLine(i18n("Program version:"), QCoreApplication::applicationVersion())
       << tableLine(i18n("Backend:"), QStringLiteral("%1 (%2)").arg(CoreBackendManager::self()->backend()->id()).arg(CoreBackendManager::self()->backend()->version()))
-      << tableLine(i18n("KDE Frameworks version:"), QStringLiteral(KXMLGUI_VERSION_STRING))
+      << tableLine(i18n("KDE Frameworks version:"), QStringLiteral(KIO_VERSION_STRING))
       << tableLine(i18n("Machine:"), unameString)
       << "</table>\n<br/>\n";
 
